@@ -56,12 +56,10 @@
 #include "scope.h"
 #include "x11.h"
 
-#ifdef SYSV
-#define bzero(s,l) memset(s, 0, l)
-#define bcopy(s,d,l) memmove(d,s,l)
-#endif
-
-static void ProcessBuffer(FD fd, unsigned char *buf, long n);
+struct TypeDef  TD[MaxTypes];
+unsigned char    RBf[2];
+unsigned char    SBf[4];
+struct ConnState    CS[StaticMaxFD];
 
 /* ************************************************************ */
 /*								*/
@@ -69,27 +67,36 @@ static void ProcessBuffer(FD fd, unsigned char *buf, long n);
 /* ************************************************************ */
 
 void
-ReportFromClient(fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+ReportFromClient(
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
-  PrintTime();
-  fprintf(stdout, "Client%s --> %4d %s\n",
-	  ClientName(fd), n, (n == 1 ? "byte" : "bytes"));
-  ProcessBuffer(fd, buf, n);
+    if (Verbose > 0)
+    {
+	if (ScopeEnabled) {
+	    PrintTime();
+	    fprintf(stdout, "Client%s --> %4ld %s\n",
+		    ClientName(fd), n, (n == 1 ? "byte" : "bytes"));
+	}
+    }
+    ProcessBuffer(fd, buf, n);
 }
 
 void
-ReportFromServer(fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+ReportFromServer (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
-  PrintTime();
-  fprintf(stdout, "\t\t\t\t\t%4d %s <-- X11 Server%s\n",
-	  n, (n == 1 ? "byte" : "bytes"), ClientName(fd));
-  ProcessBuffer(fd, buf, n);
+    if (Verbose > 0) {
+	if (ScopeEnabled) {
+	    PrintTime();
+	    fprintf(stdout, "\t\t\t\t\t%4ld %s <-- X11 Server%s\n",
+		    n, (n == 1 ? "byte" : "bytes"), ClientName(fd));
+	}
+    }
+    ProcessBuffer(fd, buf, n);
 }
 
 
@@ -98,7 +105,6 @@ ReportFromServer(fd, buf, n)
 /*								*/
 /* ************************************************************ */
 
-#include <sys/time.h>	       /* for struct timeval * */
 static long ZeroTime1 = -1;
 static long ZeroTime2 = -1;
 static struct timeval   tp;
@@ -106,7 +112,7 @@ static struct timeval   tp;
 /* print the time since we started in hundredths (1/100) of seconds */
 
 void
-PrintTime()
+PrintTime(void)
 {
   static long lastsec = 0;
   long    sec /* seconds */ ;
@@ -127,7 +133,7 @@ PrintTime()
       hsec += 100;
       sec -= 1;
     }
-  fprintf(stdout, "%2d.%02ld: ", sec, hsec);
+  fprintf(stdout, "%2ld.%02ld: ", sec, hsec);
 }
 
 /* ************************************************************ */
@@ -139,39 +145,52 @@ PrintTime()
    requests as various built-in types.  The following routines
    support the types built into X11 */
 
-long    pad (n)
-     long    n;
+long
+pad (
+    long n)
 {
   /* round up to next multiple of 4 */
   return((n + 3) & ~0x3);
 }
 
-static int  littleEndian;
-
-unsigned long    ILong (buf)
-     unsigned char   buf[];
+unsigned long
+ILong (
+    const unsigned char   buf[])
 {
+  /* check for byte-swapping */
   if (littleEndian)
     return((((((buf[3] << 8) | buf[2]) << 8) | buf[1]) << 8) | buf[0]);
   return((((((buf[0] << 8) | buf[1]) << 8) | buf[2]) << 8) | buf[3]);
 }
 
-unsigned short   IShort (buf)
-unsigned char   buf[];
+unsigned short
+IShort (
+    const unsigned char   buf[])
 {
+  /* check for byte-swapping */
   if (littleEndian)
     return (buf[1] << 8) | buf[0];
   return((buf[0] << 8) | buf[1]);
 }
 
-unsigned short   IByte (buf)
-unsigned char   buf[];
+unsigned short
+IChar2B (
+    const unsigned char   buf[])
+{
+  /* CHAR2B is like an IShort, but not byte-swapped */
+  return((buf[0] << 8) | buf[1]);
+}
+
+unsigned short
+IByte (
+    const unsigned char   buf[])
 {
   return(buf[0]);
 }
 
-Boolean IBool(buf)
-     unsigned char   buf[];
+Boolean
+IBool (
+    const unsigned char   buf[])
 {
   if (buf[0] != 0)
     return(true);
@@ -189,10 +208,10 @@ Boolean IBool(buf)
    interpret.  The following procedures provide this ability */
 
 static void
-SaveBytes(fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+SaveBytes (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
   /* check if there is enough space to hold the bytes we want */
   if (CS[fd].NumberofSavedBytes + n > CS[fd].SizeofSavedBytes)
@@ -216,9 +235,9 @@ SaveBytes(fd, buf, n)
 }
 
 static void
-RemoveSavedBytes(fd, n)
-     FD fd;
-     long    n;
+RemoveSavedBytes (
+    FD fd,
+    long    n)
 {
   /* check if all bytes are being removed -- easiest case */
   if (CS[fd].NumberofSavedBytes <= n)
@@ -246,15 +265,13 @@ RemoveSavedBytes(fd, n)
 
 /* following are the possible values for ByteProcessing */
 /* forward declarations */
-static long StartSetUpMessage(FD fd, unsigned char *buf, long n);
-static long FinishSetUpMessage(FD fd, unsigned char *buf, long n);
-static long StartRequest(FD fd, unsigned char *buf, long n);
-static long FinishRequest(FD fd, unsigned char *buf, long n);
+static long FinishSetUpMessage(FD fd, const unsigned char *buf, long n);
+static long StartRequest(FD fd, const unsigned char *buf, long n);
+static long FinishRequest(FD fd, const unsigned char *buf, long n);
 
-static long StartSetUpReply(FD fd, unsigned char *buf, long n);
-static long FinishSetUpReply(FD fd, unsigned char *buf, long n);
-static long ServerPacket(FD fd, unsigned char *buf, long n);
-static long FinishReply(FD fd, unsigned char *buf, long n);
+static long FinishSetUpReply(FD fd, const unsigned char *buf, long n);
+static long ServerPacket(FD fd, const unsigned char *buf, long n);
+static long FinishReply(FD fd, const unsigned char *buf, long n);
 
 
 /* ************************************************************ */
@@ -262,18 +279,24 @@ static long FinishReply(FD fd, unsigned char *buf, long n);
 /*								*/
 /* ************************************************************ */
 
-static void
-ProcessBuffer(fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+int littleEndian;
+
+void
+ProcessBuffer (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
-  unsigned char   *BytesToProcess;
+  const unsigned char   *BytesToProcess;
   long    NumberofUsedBytes;
 
   /* as long as we have enough bytes to do anything -- do it */
 
-  littleEndian = CS[fd].littleEndian;
+    if (Verbose > 4)
+    {
+	fprintf (stdout, "\nRead from fd %d\n", fd);
+	DumpHexBuffer (buf, n);
+    }
   while (CS[fd].NumberofSavedBytes + n >= CS[fd].NumberofBytesNeeded)
     {
       /*
@@ -323,6 +346,7 @@ ProcessBuffer(fd, buf, n)
 	the state we are in. The processing routine should return the
 	number of bytes that it actually used.
       */
+      littleEndian = CS[fd].littleEndian;
       NumberofUsedBytes = (*CS[fd].ByteProcessing)
                              (fd, BytesToProcess, CS[fd].NumberofBytesNeeded);
 
@@ -333,6 +357,7 @@ ProcessBuffer(fd, buf, n)
 
       if (NumberofUsedBytes > 0)
 	{
+	  CS[fd].NumberofBytesProcessed += NumberofUsedBytes;
 	  if (CS[fd].NumberofSavedBytes > 0)
 	    RemoveSavedBytes(fd, NumberofUsedBytes);
 	  else
@@ -345,9 +370,15 @@ ProcessBuffer(fd, buf, n)
 	}
     } /* end of while (NumberofSavedBytes + n >= NumberofBytesNeeded) */
 
+    if (Verbose > 3)
+	fprintf (stdout, "Have %ld need %ld\n",
+		 CS[fd].NumberofSavedBytes + n,
+		 CS[fd].NumberofBytesNeeded);
   /* not enough bytes -- just save the new bytes for more later */
   if (n > 0)
+  {
     SaveBytes(fd, buf, n);
+  }
   return;
 }
 
@@ -363,14 +394,68 @@ ProcessBuffer(fd, buf, n)
 */
 
 void
-StartClientConnection(fd)
-     FD fd;
+SetBufLimit (
+    FD  fd)
+{
+  int ServerFD = FDPair (fd);
+  FDinfo[ServerFD].buflimit = (CS[fd].NumberofBytesProcessed + 
+			       CS[fd].NumberofBytesNeeded);
+}
+
+void
+ClearBufLimit (
+    FD  fd)
+{
+  int ServerFD = FDPair (fd);
+  FDinfo[ServerFD].buflimit = -1;
+}
+
+static void
+StartStuff (
+    FD  fd)
+{
+  if (BreakPoint)
+  {
+    int ServerFD = FDPair (fd);
+    FDinfo[ServerFD].buflimit = (CS[fd].NumberofBytesProcessed + 
+				 CS[fd].NumberofBytesNeeded);
+    FlushFD (ServerFD);
+  }
+}
+
+static void
+FinishStuff (
+    FD fd,
+    const unsigned char	*buf,
+    long		n)
+{
+  if (BreakPoint)
+  {
+    int	ServerFD = FDPair (fd);
+    
+    FlushFD (ServerFD);
+    if (SingleStep)
+      ReadCommands ();
+    else if (BreakPoint)
+      TestBreakPoints (buf, n);
+    if (!BreakPoint)
+    {
+      FDinfo[ServerFD].buflimit = -1;
+      FlushFD (ServerFD);
+    }
+  }
+}
+
+void
+StartClientConnection (
+    FD fd)
 {
   enterprocedure("StartClientConnection");
   /* when a new connection is started, we have no saved bytes */
   CS[fd].SavedBytes = NULL;
   CS[fd].SizeofSavedBytes = 0;
   CS[fd].NumberofSavedBytes = 0;
+  CS[fd].NumberofBytesProcessed = 0;
 
   /* when a new connection is started, we have no reply Queue */
   FlushReplyQ(fd);
@@ -381,11 +466,12 @@ StartClientConnection(fd)
   /* we need 12 bytes to start a SetUp message */
   CS[fd].ByteProcessing = StartSetUpMessage;
   CS[fd].NumberofBytesNeeded = 12;
+  StartStuff (fd);
 }
 
 void
-StopClientConnection(fd)
-     FD fd;
+StopClientConnection (
+    FD fd)
 {
   enterprocedure("StopClientConnection");
   /* when a new connection is stopped, discard the old buffer */
@@ -394,10 +480,11 @@ StopClientConnection(fd)
     Free((char*)CS[fd].SavedBytes);
 }
 
-static long    StartSetUpMessage (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+long
+StartSetUpMessage (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
   unsigned short   namelength;
   unsigned short   datalength;
@@ -409,21 +496,26 @@ static long    StartSetUpMessage (fd, buf, n)
     can't process the first 12 bytes until we get all of them, so
     return zero bytes used, and increase the number of bytes needed
   */
-
+  CS[fd].littleEndian = (buf[0] == 'l');
+  CS[ServerHalf(fd)].littleEndian = CS[fd].littleEndian;
+  littleEndian = CS[fd].littleEndian;
+  
   namelength = IShort(&buf[6]);
   datalength = IShort(&buf[8]);
   CS[fd].ByteProcessing = FinishSetUpMessage;
   CS[fd].NumberofBytesNeeded = n
                                + pad((long)namelength) + pad((long)datalength);
-  debug(8,(stderr, "need %d bytes to finish startup\n",
+  debug(8,(stderr, "need %ld bytes to finish startup\n",
 	   CS[fd].NumberofBytesNeeded - n));
+  StartStuff (fd);
   return(0);
 }
 
-static long    FinishSetUpMessage (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+static long
+FinishSetUpMessage (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
   enterprocedure("FinishSetUpMessage");
   if( Raw || (Verbose > 3) )
@@ -431,43 +523,76 @@ static long    FinishSetUpMessage (fd, buf, n)
   CS[fd].littleEndian = (buf[0] == 'l');
   CS[ServerHalf(fd)].littleEndian = CS[fd].littleEndian;
   littleEndian = CS[fd].littleEndian;
-  PrintSetUpMessage(buf);
+  if (ScopeEnabled)
+    PrintSetUpMessage(buf);
 
   /* after a set-up message, we expect a string of requests */
   CS[fd].ByteProcessing = StartRequest;
   CS[fd].NumberofBytesNeeded = 4;
+  FinishStuff (fd, buf, n);
   return(n);
 }
 
+static long
+StartBigRequest (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
+{
+  enterprocedure("StartBigRequest");
 
-static long    StartRequest (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+  /* bytes 0-3 are ignored now; bytes 4-8 tell us the request length */
+  CS[fd].requestLen = ILong(&buf[4]);
+
+  CS[fd].ByteProcessing = FinishRequest;
+  CS[fd].NumberofBytesNeeded = 4 * CS[fd].requestLen;
+  debug(8,(stderr, "need %ld more bytes to finish request\n",
+	   CS[fd].NumberofBytesNeeded - n));
+  StartStuff (fd);
+  return(0);
+}
+
+static long
+StartRequest (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
-  unsigned short   requestlength;
   enterprocedure("StartRequest");
 
   /* bytes 0,1 are ignored now; bytes 2,3 tell us the request length */
-  requestlength = IShort(&buf[2]);
-
-  CS[fd].ByteProcessing = FinishRequest;
-  CS[fd].NumberofBytesNeeded = 4 * requestlength;
-  debug(8,(stderr, "need %d more bytes to finish request\n",
-	   CS[fd].NumberofBytesNeeded - n));
+  CS[fd].requestLen = IShort(&buf[2]);
+  if (CS[fd].requestLen == 0 && CS[fd].bigreqEnabled)
+  {
+    CS[fd].ByteProcessing = StartBigRequest;
+    CS[fd].NumberofBytesNeeded = 8;
+  }
+  else
+  {
+    if (CS[fd].requestLen == 0)
+      CS[fd].requestLen = 1;
+    CS[fd].ByteProcessing = FinishRequest;
+    CS[fd].NumberofBytesNeeded = 4 * CS[fd].requestLen;
+    debug(8,(stderr, "need %ld more bytes to finish request\n",
+	     CS[fd].NumberofBytesNeeded - n));
+  }
+  StartStuff (fd);
   return(0);
 }
 
 
-static long    FinishRequest (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+static long
+FinishRequest (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
   enterprocedure("FinishRequest");
-  DecodeRequest(fd, buf, n);
   CS[fd].ByteProcessing = StartRequest;
   CS[fd].NumberofBytesNeeded = 4;
+  if (ScopeEnabled)
+      DecodeRequest(fd, buf, n);
+  FinishStuff (fd, buf, n);
   return(n);
 }
 
@@ -477,14 +602,15 @@ static long    FinishRequest (fd, buf, n)
 /* ************************************************************ */
 
 void
-StartServerConnection(fd)
-     FD fd;
+StartServerConnection (
+    FD fd)
 {
   enterprocedure("StartServerConnection");
   /* when a new connection is started, we have no saved bytes */
   CS[fd].SavedBytes = NULL;
   CS[fd].SizeofSavedBytes = 0;
   CS[fd].NumberofSavedBytes = 0;
+  CS[fd].NumberofBytesProcessed = 0;
 
   /* when a new connection is started, we have no reply Queue */
   FlushReplyQ(fd);
@@ -495,8 +621,8 @@ StartServerConnection(fd)
 }
 
 void
-StopServerConnection(fd)
-     FD fd;
+StopServerConnection (
+    FD fd)
 {
   enterprocedure("StopServerConnection");
   /* when a new connection is stopped, discard the old buffer */
@@ -505,11 +631,11 @@ StopServerConnection(fd)
     Free((char *)CS[fd].SavedBytes);
 }
 
-static long
-StartSetUpReply (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+long
+StartSetUpReply (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
   unsigned short   replylength;
 
@@ -517,21 +643,22 @@ StartSetUpReply (fd, buf, n)
   replylength = IShort(&buf[6]);
   CS[fd].ByteProcessing = FinishSetUpReply;
   CS[fd].NumberofBytesNeeded = n + 4 * replylength;
-  debug(8,(stderr, "need %d bytes to finish startup reply\n",
+  debug(8,(stderr, "need %ld bytes to finish startup reply\n",
 	   CS[fd].NumberofBytesNeeded - n));
   return(0);
 }
 
 static long
-FinishSetUpReply (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+FinishSetUpReply (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
   enterprocedure("FinishSetUpReply");
   if( Raw || (Verbose > 3) )
 	DumpItem("Server Connect", fd, buf, n) ;
-  PrintSetUpReply(buf);
+  if (ScopeEnabled)
+      PrintSetUpReply(buf);
   CS[fd].ByteProcessing = ServerPacket;
   CS[fd].NumberofBytesNeeded = 32;
   return(n);
@@ -540,44 +667,61 @@ FinishSetUpReply (fd, buf, n)
 /* ************************************************************ */
 
 static long
-ErrorPacket (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+ErrorPacket (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
-  fprintf(stdout, "Error: ");
+  CS[fd].ByteProcessing = ServerPacket;
+  CS[fd].NumberofBytesNeeded = 32;
   DecodeError(fd, buf, n);
+  return(n);
+}
+
+/* FinishEvent/EventPacket: since GenericEvents may be longer than 32 bytes
+   now, mirror the FinishReply/ReplyPacket model for getting the required
+   data length to handle them. */
+
+static long
+FinishEvent (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
+{
   CS[fd].ByteProcessing = ServerPacket;
   CS[fd].NumberofBytesNeeded = 32;
+  enterprocedure("FinishEvent");
+  if (ScopeEnabled)
+    DecodeEvent(fd, buf, n);
   return(n);
 }
 
+static long
+EventPacket (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
+{
+  short Event = IByte (&buf[0]);
+  long	eventlength = 0;
+
+  CS[fd].ByteProcessing = FinishEvent;
+  CS[fd].NumberofBytesNeeded = 32;
+  if (Event == Event_Type_Generic) {
+    eventlength = ILong(&buf[4]);
+    CS[fd].NumberofBytesNeeded += (4 * eventlength);
+  }
+  debug(8,(stderr, "need %ld bytes to finish reply\n", (4 * eventlength)));
+  return(0);
+}
+
 
 static long
-EventPacket (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+ReplyPacket (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
-  DecodeEvent(fd, buf, n);
-  CS[fd].ByteProcessing = ServerPacket;
-  CS[fd].NumberofBytesNeeded = 32;
-  return(n);
-}
-
-
-static long
-ReplyPacket (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
-{
-  /*
-   B U G: 4242329
-   fix for GetImage packet, where data length is too long
-   to handle using a short field. 
-   */
-   
   long   replylength;
 
   replylength = ILong(&buf[4]);
@@ -594,15 +738,15 @@ ReplyPacket (fd, buf, n)
 
   CS[fd].ByteProcessing = FinishReply;
   CS[fd].NumberofBytesNeeded = n + 4 * replylength;
-  debug(8,(stderr, "need %d bytes to finish reply\n", (4 * replylength)));
+  debug(8,(stderr, "need %ld bytes to finish reply\n", (4 * replylength)));
   return(0);
 }
 
 static long
-ServerPacket (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+ServerPacket (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
   short   PacketType;
   enterprocedure("ServerPacket");
@@ -615,14 +759,28 @@ ServerPacket (fd, buf, n)
   return(EventPacket(fd, buf, n));
 }
 
-long    FinishReply (fd, buf, n)
-     FD fd;
-     unsigned char *buf;
-     long    n;
+long
+FinishReply (
+    FD fd,
+    const unsigned char *buf,
+    long    n)
 {
-  enterprocedure("FinishReply");
-  DecodeReply(fd, buf, n);
   CS[fd].ByteProcessing = ServerPacket;
   CS[fd].NumberofBytesNeeded = 32;
+  enterprocedure("FinishReply");
+  if (ScopeEnabled)
+    DecodeReply(fd, buf, n);
   return(n);
 }
+
+long
+GetXRequestFromName (
+    const char *name)
+{
+    long req = GetEValue (REQUEST, name);
+
+    if (req < 0)
+	req =  GetEValue (EXTENSION, name);
+
+    return req;
+}
