@@ -66,21 +66,21 @@ static void InitSetTypes(void);
 static void InitRecordTypes(void);
 static void InitValuesTypes(void);
 
-static int PrintCHAR2B(unsigned char *buf);
-static int PrintPOINT(unsigned char *buf);
-static int PrintRECTANGLE(unsigned char *buf);
-static int PrintARC(unsigned char *buf);
-static int PrintHOST(unsigned char *buf);
-static int PrintTIMECOORD(unsigned char *buf);
-static int PrintFONTPROP(unsigned char *buf);
-static int PrintCHARINFO(unsigned char *buf);
-static int PrintSEGMENT(unsigned char *buf);
-static int PrintCOLORITEM(unsigned char *buf);
-static int PrintRGB(unsigned char *buf);
-static int PrintFORMAT(unsigned char *buf);
-static int PrintSCREEN(unsigned char *buf);
-static int PrintDEPTH(unsigned char *buf);
-static int PrintVISUALTYPE(unsigned char *buf);
+static int PrintCHAR2B(const unsigned char *buf);
+static int PrintPOINT(const unsigned char *buf);
+static int PrintRECTANGLE(const unsigned char *buf);
+static int PrintARC(const unsigned char *buf);
+static int PrintHOST(const unsigned char *buf);
+static int PrintTIMECOORD(const unsigned char *buf);
+static int PrintFONTPROP(const unsigned char *buf);
+static int PrintCHARINFO(const unsigned char *buf);
+static int PrintSEGMENT(const unsigned char *buf);
+static int PrintCOLORITEM(const unsigned char *buf);
+static int PrintRGB(const unsigned char *buf);
+static int PrintFORMAT(const unsigned char *buf);
+static int PrintSCREEN(const unsigned char *buf);
+static int PrintDEPTH(const unsigned char *buf);
+static int PrintVISUALTYPE(const unsigned char *buf);
 
 /*
   To initialize for the X11 protocol, we need to create data structures
@@ -111,7 +111,7 @@ static int PrintVISUALTYPE(unsigned char *buf);
 */
 
 void
-InitializeX11()
+InitializeX11 (void)
 {
   InitReplyQ();
 
@@ -122,6 +122,105 @@ InitializeX11()
   InitRecordTypes();
 }
 
+#define HASH_SIZE   997
+
+ValuePtr    buckets[HASH_SIZE];
+
+#define HASH(key)   ((key) % HASH_SIZE)
+
+ValuePtr
+GetValueRec (
+    unsigned long   key)
+{
+    ValuePtr	*bucket, value;
+
+    bucket = &buckets[HASH(key)];
+    for (value = *bucket; value; value = value->next)
+    {
+	if (value->key == key)
+	    return value;
+    }
+    return NULL;
+}
+
+void
+CreateValueRec (
+    unsigned long   key,
+    int		    size,
+    const unsigned long   *def)
+{
+    ValuePtr	*bucket, value;
+    int		i;
+    
+    bucket = &buckets[HASH(key)];
+    value = (ValuePtr) malloc (sizeof (ValueRec) + size * sizeof (unsigned long));
+    if (!value)
+	return;
+    value->values = (unsigned long *) (value + 1);
+    for (i = 0; i < size; i++)
+	value->values[i] = ILong((const unsigned char *) (def + i));
+    value->size = size;
+    value->key = key;
+    value->next = *bucket;
+    *bucket = value;
+}
+
+void
+DeleteValueRec (
+    unsigned long   key)
+{
+    ValuePtr	*bucket, value;
+
+    for (bucket = &buckets[HASH(key)]; value = *bucket; bucket = &value->next)
+    {
+	if (value->key == key)
+	{
+	    *bucket = value->next;
+	    free (value);
+	    return;
+	}
+    }
+}
+
+void
+SetValueRec (
+    unsigned long   key,
+    const unsigned char   *control,
+    short	    clength,
+    short	    ctype,
+    const unsigned char   *values)
+{
+    long    cmask;
+    struct ValueListEntry  *p;
+    ValuePtr	value;
+    int		i;
+
+    value = GetValueRec (key);
+    if (!value)
+	return;
+    /* first get the control mask */
+    if (clength == 1)
+	cmask = IByte(control);
+    else if (clength == 2)
+	cmask = IShort(control);
+    else
+	cmask = ILong(control);
+
+    /* now if it is zero, ignore and return */
+    if (cmask == 0)
+	return;
+    /* there are bits in the controlling bitmask, figure out which */
+    /* the ctype is a set type, so this code is similar to PrintSET */
+    for (p = TD[ctype].ValueList, i = 0; p != NULL; p = p->Next, i++)
+    {
+	if ((p->Value & cmask) != 0)
+	{
+	    memcpy (&value->values[i], values, sizeof (unsigned long));
+	    values += 4;
+	}
+    }
+}
+
 /* ************************************************************ */
 /*								*/
 /*								*/
@@ -129,12 +228,12 @@ InitializeX11()
 
 /* define the various types */
 
-static TYPE 
-DefineType(typeid, class, name, printproc)
-     short   typeid;
-     short   class;
-     char   *name;
-     int   (*printproc)(unsigned char *);
+TYPE
+DefineType (
+    short   typeid,
+    short   class,
+    const char   *name,
+    int   (*printproc)(const unsigned char *))
 {
   TD[typeid].Name = name;
   TD[typeid].Type = class;
@@ -146,11 +245,11 @@ DefineType(typeid, class, name, printproc)
 /* ************************************************************ */
 /* define an Enumerated Value (or a Set Value) */
 
-static void
-DefineEValue(type, value, name)
-     TYPE type;
-     long    value;
-     char   *name;
+void
+DefineEValue(
+    TYPE type,
+    long    value,
+    const char   *name)
 {
   struct ValueListEntry  *p;
 
@@ -177,6 +276,25 @@ DefineEValue(type, value, name)
     }
 }
 
+long
+GetEValue (
+    short   typeid,
+    const char    *name)
+{
+    TYPE    p;
+    struct ValueListEntry   *v;
+
+    if (typeid < 0 || MaxTypes <= typeid)
+	return -1;
+    p = &TD[typeid];
+    if (!p)
+	return -2;
+    for (v = p->ValueList; v; v = v->Next)
+	if (!strcmp (name, v->Name))
+	    return v->Value;
+    return -3;
+}
+
 /* ************************************************************ */
 /* a Values list is like an enumerated Value, but has a type and length
    in addition to a value and name.  It is used to print a Values List */
@@ -185,8 +303,9 @@ DefineEValue(type, value, name)
    we have an associated value.  We need to know the length and type of the
    associated value for each bit */
 
-static void
-DefineValues(TYPE type, long value, short length, short ctype, char *name)
+void
+DefineValues(TYPE type, long value, short length, short ctype,
+	     const char *name)
 {
   struct ValueListEntry  *p;
 
@@ -219,7 +338,7 @@ DefineValues(TYPE type, long value, short length, short ctype, char *name)
 /* ************************************************************ */
 
 static void
-InitBuiltInTypes()
+InitBuiltInTypes (void)
 {
   (void) DefineType(INT8, BUILTIN, "INT8", PrintINT8);
   (void) DefineType(INT16, BUILTIN, "INT16", PrintINT16);
@@ -264,7 +383,7 @@ InitBuiltInTypes()
 /* ************************************************************ */
 
 static void
-InitEnumeratedTypes()
+InitEnumeratedTypes (void)
 {
   TYPE p;
 
@@ -485,6 +604,7 @@ InitEnumeratedTypes()
   DefineEValue(p, 32L, "ColormapNotify");
   DefineEValue(p, 33L, "ClientMessage");
   DefineEValue(p, 34L, "MappingNotify");
+  DefineEValue(p, 35L, "GenericEvent");
 
 
   p = DefineType(BITGRAVITY, ENUMERATED, "BITGRAVITY", (PrintProcType) PrintENUMERATED);
@@ -521,6 +641,7 @@ InitEnumeratedTypes()
   DefineEValue(p, 0L, "Internet");
   DefineEValue(p, 1L, "DECnet");
   DefineEValue(p, 2L, "Chaos");
+  DefineEValue(p, 5L, "ServerInterpreted");
   DefineEValue(p, 6L, "InternetV6");
   DefineEValue(p, 252L, "LocalHost");
   DefineEValue(p, 253L, "Kerberos5");
@@ -757,6 +878,8 @@ InitEnumeratedTypes()
   DefineEValue(p, 3L, "PseudoColor");
   DefineEValue(p, 4L, "TrueColor");
   DefineEValue(p, 5L, "DirectColor");
+
+  p = DefineType(EXTENSION, ENUMERATED, "EXTENSION", (PrintProcType) PrintENUMERATED);
 }
 
 /* ************************************************************ */
@@ -765,7 +888,7 @@ InitEnumeratedTypes()
 /* ************************************************************ */
 
 static void
-InitSetTypes()
+InitSetTypes (void)
 {
   TYPE p;
 
@@ -869,8 +992,8 @@ InitSetTypes()
 /* Print Routines for builtin record types */
 
 static int
-PrintCHAR2B(buf)
-     unsigned char *buf;
+PrintCHAR2B (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 1, CARD8, "byte1");
   PrintField(buf, 1, 1, CARD8, "byte2");
@@ -878,8 +1001,8 @@ PrintCHAR2B(buf)
 }
 
 static int
-PrintPOINT(buf)
-     unsigned char *buf;
+PrintPOINT (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 2, INT16, "x");
   PrintField(buf, 2, 2, INT16, "y");
@@ -887,8 +1010,8 @@ PrintPOINT(buf)
 }
 
 static int
-PrintRECTANGLE(buf)
-     unsigned char *buf;
+PrintRECTANGLE (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 2, INT16, "x");
   PrintField(buf, 2, 2, INT16, "y");
@@ -898,8 +1021,8 @@ PrintRECTANGLE(buf)
 }
 
 static int
-PrintARC(buf)
-     unsigned char *buf;
+PrintARC (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 2, INT16, "x");
   PrintField(buf, 2, 2, INT16, "y");
@@ -911,8 +1034,8 @@ PrintARC(buf)
 }
 
 static int
-PrintHOST(buf)
-     unsigned char *buf;
+PrintHOST (
+    const unsigned char *buf)
 {
   short   n;
   PrintField(buf, 0, 1, HOSTFAMILY, "family");
@@ -940,6 +1063,15 @@ PrintHOST(buf)
     }
 #endif
 
+    case 5: /* ServerInterpreted */
+    {
+	int i;
+	for (i = 0 ; buf[i + 4] != 0 ; i++) { /* empty loop */ }
+	PrintString8(&buf[4], i, "type");
+	PrintString8(&buf[i+5], n - i - 1, "value");
+	break;
+    }
+
     case 254:
 	PrintString8(&buf[4], n, "address");
 	break;
@@ -951,8 +1083,8 @@ PrintHOST(buf)
 }
 
 static int
-PrintTIMECOORD(buf)
-     unsigned char *buf;
+PrintTIMECOORD (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 4, TIMESTAMP, "time");
   PrintField(buf, 4, 2, CARD16, "x");
@@ -961,8 +1093,8 @@ PrintTIMECOORD(buf)
 }
 
 static int
-PrintFONTPROP(buf)
-     unsigned char *buf;
+PrintFONTPROP (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 4, ATOM, "name");
   PrintField(buf, 4, 4, INT32, "value");
@@ -970,8 +1102,8 @@ PrintFONTPROP(buf)
 }
 
 static int
-PrintCHARINFO(buf)
-     unsigned char *buf;
+PrintCHARINFO (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 2, INT16, "left-side-bearing");
   PrintField(buf, 2, 2, INT16, "right-side-bearing");
@@ -983,8 +1115,8 @@ PrintCHARINFO(buf)
 }
 
 static int
-PrintSEGMENT(buf)
-     unsigned char *buf;
+PrintSEGMENT (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 2, INT16, "x1");
   PrintField(buf, 2, 2, INT16, "y1");
@@ -994,8 +1126,8 @@ PrintSEGMENT(buf)
 }
 
 static int
-PrintCOLORITEM(buf)
-     unsigned char *buf;
+PrintCOLORITEM (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 4, CARD32, "pixel");
   PrintField(buf, 4, 2, CARD16, "red");
@@ -1006,8 +1138,8 @@ PrintCOLORITEM(buf)
 }
 
 static int
-PrintRGB(buf)
-     unsigned char *buf;
+PrintRGB (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 2, CARD16, "red");
   PrintField(buf, 2, 2, CARD16, "green");
@@ -1016,8 +1148,8 @@ PrintRGB(buf)
 }
 
 static int
-PrintFORMAT(buf)
-     unsigned char *buf;
+PrintFORMAT (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 1, CARD8, "depth");
   PrintField(buf, 1, 1, CARD8, "bits-per-pixel");
@@ -1026,8 +1158,8 @@ PrintFORMAT(buf)
 }
 
 static int
-PrintSCREEN(buf)
-     unsigned char *buf;
+PrintSCREEN (
+    const unsigned char *buf)
 {
   short   n /* number of elements in List of DEPTH */ ;
   long    m /* length (in bytes) of List of DEPTH */ ;
@@ -1054,8 +1186,8 @@ PrintSCREEN(buf)
 }
 
 static int
-PrintDEPTH(buf)
-     unsigned char *buf;
+PrintDEPTH (
+    const unsigned char *buf)
 {
   short   n /* number of elements in List of VISUALTYPE */ ;
   short   m /* length (in bytes) of List of VISUALTYPE */ ;
@@ -1068,8 +1200,8 @@ PrintDEPTH(buf)
 }
 
 static int
-PrintVISUALTYPE(buf)
-     unsigned char *buf;
+PrintVISUALTYPE (
+    const unsigned char *buf)
 {
   PrintField(buf, 0, 4, VISUALID, "visual-id");
   PrintField(buf, 4, 1, COLORCLASS, "class");
@@ -1084,7 +1216,7 @@ PrintVISUALTYPE(buf)
 /* ************************************************************ */
 
 static void
-InitRecordTypes()
+InitRecordTypes (void)
 {
   (void) DefineType(CHAR2B, RECORD, "CHAR2B", PrintCHAR2B);
   (void) DefineType(POINT, RECORD, "POINT", PrintPOINT);
@@ -1111,7 +1243,7 @@ InitRecordTypes()
 /* ************************************************************ */
 
 static void
-InitValuesTypes()
+InitValuesTypes (void)
 {
   TYPE p;
 
